package com.adc.da.bs.customized.rest;

import com.adc.da.bs.customized.enums.CustomizedServiceEnum;
import com.adc.da.bs.customized.service.CustomizedService;
import com.adc.da.bs.customized.vo.CustomizedServiceQueryVO;
import com.adc.da.util.http.ResponseMessage;
import com.adc.da.util.http.Result;
import com.adc.da.util.utils.StringUtils;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletRequest;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @author AnYuShi
 * @description 定制服务
 * @date create at 16:29 2018/3/20
 * @modified by
 */
@RestController
@RequestMapping(value = "/${restPath}/customized")
@Api(description = "定制服务请求")
public class CustomizedController {

    @Autowired
    private CustomizedService customizedService;

    /**
     * 新增定制服务请求
     *
     * @param customizedServiceQueryVO
     * @return java.util.Map<java.lang.String                               ,                               java.lang.Object>
     * @author 韩磊
     * @date 2019/3/21 17:19
     */
    @ApiOperation(value = "新增定制服务请求")
    @PostMapping
    public ResponseMessage createCustomizedService(HttpServletRequest request,
            CustomizedServiceQueryVO customizedServiceQueryVO) {
        Map<String, Object> usermap = (Map<String, Object>) request.getSession().getAttribute("usermap");
        Map<String, Object> remap = new HashMap<>();
        //判断登录人
        if (usermap == null || usermap.isEmpty()) {
            return Result.error("请先登录系统后再进行申请.");
        }
        if(StringUtils.isEmpty(customizedServiceQueryVO.getPhoneNum())){
            return Result.error("请填写手机号.");
        }
        if(StringUtils.isEmpty(customizedServiceQueryVO.getContacter())){
            return Result.error("请填写联系人.");
        }
        if(StringUtils.isEmpty(customizedServiceQueryVO.getServiceType())){
            return Result.error("请重新选择服务类型.");
        }
        if(StringUtils.isEmpty(customizedServiceQueryVO.getContent())){
            return Result.error("请填写定制内容.");
        }
        //判断传参是否正确
        if (StringUtils.isNoneBlank(customizedServiceQueryVO.getServiceType())) {
            List<String> serviceTypeList = Arrays.asList(customizedServiceQueryVO.getServiceType().split(","));
            for (String type : serviceTypeList) {
                if (null == CustomizedServiceEnum.getServiceEnumByValue(Integer.parseInt(type))) {
                    return Result.error("请选择正确的服务类型。");
                }
            }
        }

        int n = customizedService.createCustomizedService(customizedServiceQueryVO, usermap);
        if (0 != n) {
            return Result.success();
        } else {
            return Result.error("系统忙，请稍后再试。");
        }
    }

    /**
     * 定制服务请求 列表
     *
     * @return java.util.Map<java.lang.String                               ,                               java.lang.Object>
     * @author 韩磊
     * @date 2019/3/21 17:19
     */
    @ApiOperation(value = "定制服务请求 查询历史")
    @GetMapping
    @ResponseBody
    public ResponseMessage getCustomizedServiceHistory(HttpServletRequest request) {
        Map<String, Object> usermap = (Map<String, Object>) request.getSession().getAttribute("usermap");
        Map<String, Object> remap = new HashMap<>();
        //判断登录人
        if (usermap == null || usermap.isEmpty()) {
            return Result.error("请先登录系统后再进行申请.");
        }
        Map<String, Object> data = customizedService.getCustomizedServiceHistory(usermap);
        return Result.success(data);
    }
}
