package com.adc.da.bs.manage.dao;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.persistence.EntityManager;
import javax.persistence.Query;
import javax.transaction.Transactional;

import org.apache.commons.lang3.StringUtils;
import org.hibernate.transform.Transformers;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.jpa.repository.Modifying;
import org.springframework.stereotype.Component;

import com.adc.da.bs.manage.entity.WorkManageEntity;
import com.adc.da.util.http.PageParam;
/**
 * 4.3.	办事指南管理Dao
 * @author LuZhiPeng
 *
 */
@Component
public class WorkManageDao {
	protected final EntityManager entityManager;
    @Autowired
    public WorkManageDao(EntityManager entityManager) {
        super();
        this.entityManager = entityManager;
    }
    /**
     * 分页并按状态（降序：status=Y为有效，status=N为无效）上传日期（降序）显示所有记录
     * @param page
     * @return
     */
    public List<Map<String,Object>> findAllByPage(PageParam page){
    	StringBuffer sql = new StringBuffer(" SELECT ROWNUM RN ,T.* FROM ( ");
    	sql.append("SELECT W.ID, W.NAME, W.TYPE_NUM, to_char(W.UPLOAD_DATE,'yyyy-MM-dd') UPLOAD_DATE, W.STATUS, W.URL,");
    	sql.append("W.DEL_NUM, to_char(W.DELETE_DATE,'yyyy-MM-dd HH24:mi:ss'), W.CREATOR, W.VIEW_COUNT, W.DOWNLOAD_COUNT ");
    	sql.append("FROM NERDS.WORK_MANAGE W WHERE W.DEL_NUM='0' ");    	
    	sql.append("ORDER BY W.STATUS DESC, to_char(W.UPLOAD_DATE,'yyyy-MM-dd HH24:mi:ss') DESC");
    	sql.append(" ) T where ROWNUM<5");
    	Query query = entityManager.createNativeQuery(sql.toString());
    	query.unwrap(org.hibernate.SQLQuery.class).setResultTransformer(Transformers.ALIAS_TO_ENTITY_MAP);
    	query.setFirstResult(page.getOffset());
        query.setMaxResults(page.getLimit());
        List<Map<String, Object>> list = query.getResultList();
        return list;
    }
    /**
     * 按条件查询出所有的行数
     * @param type  查询类型
     * @param name  查询记录标题名称
     * @return
     */
    
    public Long findTotalCount(PageParam page,String type, String name) {
    	StringBuffer sql = new StringBuffer();
    	sql.append("SELECT COUNT(*) FROM NERDS.WORK_MANAGE W WHERE W.DEL_NUM='0' ");
    	if(StringUtils.isNotEmpty(type)){      //判断是否输入类型
    		sql.append("AND W.TYPE_NUM LIKE concat(concat('%',:type),'%') ");   
    	}
    	if(StringUtils.isNotEmpty(name)) {    //判断是否输入记录标题名称
    		sql.append("AND W.NAME LIKE concat(concat('%',:name),'%') ");
    	}
    	if(StringUtils.isNotEmpty(page.getStartTime())) {            //判断是否输入起始时间
    		sql.append("AND to_char(W.UPLOAD_DATE,'yyyy-MM-dd HH24:mi:ss')>='"+page.getStartTime()+"'");
    	}
    	if(StringUtils.isNotEmpty(page.getEndTime())) {            //判断是否输入结束时间
    		sql.append("AND to_char(W.UPLOAD_DATE,'yyyy-MM-dd HH24:mi:ss')<='"+page.getEndTime()+"'");
    	}
    	
    	Query query = entityManager.createNativeQuery(sql.toString());
    	if(StringUtils.isNotEmpty(type)){
         	query.setParameter("type", type);
         }
         if(StringUtils.isNotEmpty(name)){
         	query.setParameter("name", name);
         }
         /*if(StringUtils.isNotEmpty(page.getStartTime())){
         	query.setParameter("startTime", page.getStartTime());
         }
         if(StringUtils.isNotEmpty(page.getEndTime())){
         	query.setParameter("endTime", page.getEndTime());
         }*/
         
    	Long count = Long.parseLong(query.getSingleResult().toString());
        return count;
    }
    /**
     * 条件查询记录并分页
     * @param page  页
     * @param type  查询类型
     * @param name  查询记录标题名称
     * @return
     */   
    public List<Map<String,Object>> findListByPage(PageParam page,String type, String name){
    	StringBuffer sql = new StringBuffer(" SELECT ROWNUM RN ,T.* FROM ( ");
    	sql.append("SELECT W.ID, W.NAME, W.TYPE_NUM, to_char(W.UPLOAD_DATE,'yyyy-MM-dd') UPLOAD_DATE, W.STATUS, W.URL,");
    	sql.append("W.DEL_NUM,W.CREATOR, W.VIEW_COUNT, W.DOWNLOAD_COUNT ");
    	sql.append("FROM NERDS.WORK_MANAGE W WHERE W.DEL_NUM='0' ");    	
    	if(StringUtils.isNotEmpty(type)){      //判断是否输入类型
    		sql.append("AND W.TYPE_NUM LIKE concat(concat('%',:type),'%') ");   
    	}
    	if(StringUtils.isNotEmpty(name)) {    //判断是否输入记录标题名称
    		sql.append("AND W.NAME LIKE concat(concat('%',:name),'%') ");
    	}
    	if(StringUtils.isNotEmpty(page.getStartTime())) {            //判断是否输入起始时间
    		sql.append("AND to_char(W.UPLOAD_DATE,'yyyy-MM-dd HH24:mi:ss')>='"+page.getStartTime()+"'");
    	}
    	if(StringUtils.isNotEmpty(page.getEndTime())) {            //判断是否输入结束时间
    		sql.append("AND to_char(W.UPLOAD_DATE,'yyyy-MM-dd HH24:mi:ss')<='"+page.getEndTime()+"'");
    	}
    	sql.append("ORDER BY W.STATUS DESC, to_char(W.UPLOAD_DATE,'yyyy-MM-dd HH24:mi:ss') DESC");
    	sql.append(" ) T ");
    	 Query query = entityManager.createNativeQuery(sql.toString());
         query.unwrap(org.hibernate.SQLQuery.class).setResultTransformer(Transformers.ALIAS_TO_ENTITY_MAP);
         if(StringUtils.isNotEmpty(type)){
         	query.setParameter("type", type);
         }
         if(StringUtils.isNotEmpty(name)){
         	query.setParameter("name", name);
         }
        /* if(StringUtils.isNotEmpty(page.getStartTime())){
         	query.setParameter("startTime", page.getStartTime());
         }
         if(StringUtils.isNotEmpty(page.getEndTime())){
         	query.setParameter("endTime", page.getEndTime());
         }*/
         
         
         query.setFirstResult(page.getOffset());
         query.setMaxResults(page.getLimit());
         
         
         List<Map<String, Object>> list = query.getResultList();
         return list;    	
    }
    
    /**
     * 增加新记录
     * @param workManageEntity
     * @return
     */
    public Map<String,Object> insertWork(WorkManageEntity workManageEntity){
    	String idSql = "SELECT NERDS.NUMBER_SEQ.nextval from dual";
    	Query idQuery = entityManager.createNativeQuery(idSql);
        Integer id = Integer.parseInt(idQuery.getSingleResult().toString());
    	Map<String,Object> res = new HashMap<>();
    	//创建上传时间，并转化为字符串类型
    	Date date=new Date();
		SimpleDateFormat sf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
		String s=sf.format(date);	
		if(workManageEntity.getUploadDate()==null){
			workManageEntity.setUploadDate(s);
		}
    	StringBuffer sql = new StringBuffer();
      	sql.append("INSERT INTO NERDS.WORK_MANAGE (ID,NAME,TYPE_NUM,UPLOAD_DATE,STATUS,URL,DEL_NUM,CREATOR,VIEW_COUNT,DOWNLOAD_COUNT) VALUES ");
    	sql.append("("+id+",");
    	sql.append("'"+workManageEntity.getName()+"',");
    	sql.append("'"+workManageEntity.getTypeNum()+"',");
//    	sql.append(""+"to_date ( '"+s+"', 'YYYY-MM-DD HH24:MI:SS' )"+",");
    	sql.append(""+"to_date ( '"+workManageEntity.getUploadDate()+"', 'YYYY-MM-DD HH24:MI:SS' )"+",");
    	sql.append("'"+workManageEntity.getStatus()+"',");
    	sql.append("'"+workManageEntity.getUrl()+"',");
    	sql.append("'0',");
    	//sql.append(""+null+",");
    	sql.append("'"+workManageEntity.getCreator()+"',");
    	sql.append(""+workManageEntity.getViewCount()+",");
    	sql.append(""+workManageEntity.getDownloadCount()+")");
    	
    	try {
            Query query = entityManager.createNativeQuery(sql.toString());

            //query.setParameter(1, workManageEntity.getId());
			/*
			 * query.setParameter(2, workManageEntity.getName()); 
			 * query.setParameter(3,workManageEntity.getTypeNum());
			 *  query.setParameter(4, new Date());
			 * query.setParameter(5, workManageEntity.getStatus()); 
			 * query.setParameter(6, workManageEntity.getUrl());
			 * query.setParameter(7,workManageEntity.getDelNum()); 
			 * query.setParameter(8, null);
			 * query.setParameter(9, workManageEntity.getCreator());
			 * query.setParameter(10,workManageEntity.getViewCount()); 
			 * query.setParameter(11, workManageEntity.getDownloadCount());
			 */
            if (query.executeUpdate() == 1){
                res.put("success", "1");
                res.put("msg", "保存成功");
            }else{
            	res.put("success", "0");
                res.put("msg", "保存失败");
            }
        } catch (Exception e) {
            res.put("success", "0");
            res.put("msg", e.toString());
            e.printStackTrace();
        }
        return res;
    }
    /**
            * 增加浏览数
     * @param id
     * @return
     */
    public boolean changeViewCountById(Integer id) {
        String updateViewSql = " UPDATE NERDS.WORK_MANAGE SET VIEW_COUNT = (VIEW_COUNT+1) WHERE ID = ? ";
        Query updateQuery = entityManager.createNativeQuery(updateViewSql);
        updateQuery.setParameter(1, id);
        if(updateQuery.executeUpdate() == 1){
            return true;
        }
        return false;
    }
    /**
            * 增加下载数
     * @param id
     * @return
     */
    @Transactional
    @Modifying
    public boolean changeDownloadCountById(Integer id) {
        String updateDownloadSql = " UPDATE NERDS.WORK_MANAGE SET DOWNLOAD_COUNT = (DOWNLOAD_COUNT+1) WHERE ID = ? ";
        Query updateQuery = entityManager.createNativeQuery(updateDownloadSql);
        updateQuery.setParameter(1, id);
        if(updateQuery.executeUpdate() == 1){
            return true;
        }
        return false;
    }
    
    /**
     * 逻辑删除记录：NERDS.WORK_MANAGE.DEL_NUM=1为删除，NERDS.WORK_MANAGE.DEL_NUM=0为可使用
     * @param id
     * @return
     */
    @Transactional
    @Modifying
    public boolean deleteWork(Integer id) {
    	Date date=new Date();
		SimpleDateFormat sf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
		String s=sf.format(date);
    	StringBuffer deleteSql = new StringBuffer();
    	deleteSql.append(" UPDATE NERDS.WORK_MANAGE SET DEL_NUM = '1',");
    	deleteSql.append("DELETE_DATE="+"to_date ( '"+s+"', 'YYYY-MM-DD HH24:MI:SS' ) ");
    	deleteSql.append(""+"WHERE ID = '"+id+"'");
		Query query = entityManager.createNativeQuery(deleteSql.toString());
        if(query.executeUpdate() == 1 ){
            return true;
        }
        return false;
	}
	/**   
	 * @Title: workmanageIndex   
	 * @Description: TODO(这里用一句话描述这个方法的作用)    
	 * @param @return    设定文件   
	 * @return List<Map<String,Object>>    返回类型   
	 * @author ADC_ZH 
	 * @date 上午10:23:27 
	*/   
	public List<Map<String, Object>> workmanageIndex() {
		StringBuffer bf = new StringBuffer();
		bf.append(" SELECT  *  FROM ");
		bf.append("	(SELECT W.* , row_number() over (partition by TYPE_NUM order by id desc) rn  FROM NERDS.WORK_MANAGE W WHERE W.DEL_NUM='0' ) ");
		bf.append(" WHERE  rn<3 ORDER BY TYPE_NUM");
		Query query = entityManager.createNativeQuery(bf.toString());
		query.unwrap(org.hibernate.SQLQuery.class).setResultTransformer(Transformers.ALIAS_TO_ENTITY_MAP);  
		List<Map<String, Object>> list = query.getResultList();
		return list;
	}
	/**   
	 * @Title: workmanageIndexCountList   
	 * @Description: TODO(这里用一句话描述这个方法的作用)    
	 * @param @return    设定文件   
	 * @return List<Map<String,Object>>    返回类型   
	 * @author ADC_ZH 
	 * @date 上午10:23:32 
	*/   
	public List<Map<String, Object>> workmanageIndexCountList() {
		StringBuffer bf = new StringBuffer();
		bf.append("	SELECT TYPE_NUM,COUNT(*) COU FROM  NERDS.WORK_MANAGE ");
		bf.append("	WHERE DEL_NUM='0'");
		bf.append("	GROUP BY TYPE_NUM  ");
		Query query = entityManager.createNativeQuery(bf.toString());
		query.unwrap(org.hibernate.SQLQuery.class).setResultTransformer(Transformers.ALIAS_TO_ENTITY_MAP); 
		List<Map<String, Object>> list = query.getResultList();
		return list;
	}

    }
	
    	

