package com.adc.da.bs.param.dao;

import com.adc.da.bs.param.entity.MbxmEO;
import com.adc.da.bs.param.entity.MbxmVO;
import com.adc.da.bs.param.entity.NerdsInfoEO;
import com.adc.da.util.utils.StringUtils;
import com.adc.da.util.utils.UUID;
import org.hibernate.transform.Transformers;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.jpa.repository.Modifying;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import javax.persistence.EntityManager;
import javax.persistence.Query;
import java.sql.Clob;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Component
public class MbDao {
    protected final EntityManager entityManager;
    @Autowired
    public MbDao(EntityManager entityManager) {
        super();
        this.entityManager = entityManager;
    }
    /**
     * 获取模板列表
     */
    @SuppressWarnings("unchecked")
    public List<MbxmEO> queryMbxmList(String type) {
        // 默认查公告整车参数
        if(StringUtils.isEmpty(type)){
            type="ZC";
        }
        StringBuilder sql = new StringBuilder("SELECT ID,NAME,ZK_NAME FROM NERDS.GGCS_MBXM WHERE TYPE = ? ORDER BY TO_NUMBER(ID) ");
        Query query = entityManager.createNativeQuery(sql.toString());
        query.unwrap(org.hibernate.SQLQuery.class).setResultTransformer(Transformers.ALIAS_TO_ENTITY_MAP);
        query.setParameter(1,type);

        List<Map<String, Object>> list = query.getResultList();

        List<MbxmEO> mbxms = new ArrayList<>();
        for (Map<String, Object> resultMap : list) {
            MbxmEO mbxm = new MbxmEO();
            if (null != resultMap.get("ID")) {
                mbxm.setId(resultMap.get("ID").toString());
            }
            if (null != resultMap.get("NAME")) {
                mbxm.setName(resultMap.get("NAME").toString());
            }
            if (null != resultMap.get("ZK_NAME")) {
                mbxm.setZkName(resultMap.get("ZK_NAME").toString());
            }
            mbxm.setType(type);
            mbxms.add(mbxm);
        }
        return mbxms;
    }

    /**
     * 获取模板列表
     */
    @SuppressWarnings("unchecked")
    public List<Map<String, Object>> findMbInfoById(String mbid) {
        // 默认查公告整车参数
        StringBuilder sql = new StringBuilder("SELECT * FROM NERDS.GGCS_MB WHERE ID = ? ORDER BY ID,CREATE_TIME ");
        Query query = entityManager.createNativeQuery(sql.toString());
        query.unwrap(org.hibernate.SQLQuery.class).setResultTransformer(Transformers.ALIAS_TO_ENTITY_MAP);
        query.setParameter(1, mbid);
        List<Map<String, Object>> list = query.getResultList();

        for(Map<String, Object> data:list){
            for(String key:data.keySet()){
                if(data.get(key) instanceof Clob){
                    Clob clob  = (Clob) data.get(key);
                    try {
                        data.put(key, clob.getSubString((long)1,(int)clob.length()));
                    } catch (SQLException e) {
                        e.printStackTrace();
                    }
                }
            }
        }

        return list;
    }
    /**
     * 创建模板项目
     */
    @Transactional
    public void createMbxm(MbxmVO mbxmVO) {

        // 整车数据情况
        StringBuilder sb = new StringBuilder();
        if (mbxmVO.getZcList().size() >= 1) {
            createMbxm(mbxmVO, "ZC");
        }

        if (mbxmVO.getDpList().size() >= 1) {
            createMbxm(mbxmVO, "DP");
        }

        if (mbxmVO.getBaList().size() >= 1) {
            createMbxm(mbxmVO, "BA");
        }

    }

    private void createMbxm(MbxmVO mbxmVO, String type) {
        ArrayList<String> paramList = new ArrayList<String>();
        ArrayList<String> zkParamList = new ArrayList<String>();
        StringBuilder sb = new StringBuilder();
        StringBuilder sbzk = new StringBuilder();

        if ("ZC".equals(type)) {
            paramList = mbxmVO.getZcList();
            zkParamList = mbxmVO.getZcZkList();
        } else if ("DP".equals(type)) {
            paramList = mbxmVO.getDpList();
            zkParamList = mbxmVO.getDpZkList();
        } else if ("BA".equals(type)) {
            paramList = mbxmVO.getBaList();
            zkParamList = mbxmVO.getBaZkList();
        }

        for (String mbxm : paramList) {
            if (sb.length() > 0) {
                sb.append(",");
            }
            sb.append(mbxm);
        }

        for (String mbzkxm : zkParamList) {
            if (sbzk.length() > 0) {
                sbzk.append(",");
            }
            sbzk.append(mbzkxm);
        }
        String insSql = " INSERT INTO NERDS.GGCS_MB(ID,NAME,PARAMVALUE,CREATE_TIME,QYID,TYPE,ZKPARAMVALUE)  VALUES (?, ?, ?, sysdate, ?, ?, ?)";
        Query insQuery = entityManager.createNativeQuery(insSql);
        insQuery.setParameter(1, UUID.randomUUID());
        insQuery.setParameter(2, mbxmVO.getMbName());
        insQuery.setParameter(3, sb.toString());
        insQuery.setParameter(4, mbxmVO.getQyid());
        insQuery.setParameter(5, type);
        insQuery.setParameter(6, sbzk.toString());
        insQuery.executeUpdate();
    }

    public List<Map<String, Object>> findMbListByQyid(String qyid, String mbmc) {

        StringBuilder sql = new StringBuilder("SELECT a.*,case when type='ZC' then 1 when type='DP' then 2 else 3 end  as sortAsc FROM NERDS.GGCS_MB a WHERE QYID = ? ");

        if (null != mbmc && mbmc != null && !"".equals(mbmc)) {
            sql.append(" AND NAME like ? ");
        }

        sql.append(" ORDER BY name, sortAsc ");

        Query query = entityManager.createNativeQuery(sql.toString());
        query.unwrap(org.hibernate.SQLQuery.class).setResultTransformer(Transformers.ALIAS_TO_ENTITY_MAP);
        query.setParameter(1, qyid);

        if (null != mbmc && mbmc != null && !"".equals(mbmc)){
            query.setParameter(2, "%"+mbmc+"%");
        }

        List<Map<String, Object>> list = query.getResultList();

        for(Map<String, Object> data:list){
            for(String key:data.keySet()){
                if(data.get(key) instanceof Clob){
                    Clob clob  = (Clob) data.get(key);
                    try {
                        data.put(key, clob.getSubString((long)1,(int)clob.length()));
                    } catch (SQLException e) {
                        e.printStackTrace();
                    }
                }
            }
        }

        return list;
    }

    public List<Map<String, Object>> findZkBySql(String querySql) {

        Query query = entityManager.createNativeQuery(querySql);
        query.unwrap(org.hibernate.SQLQuery.class).setResultTransformer(Transformers.ALIAS_TO_ENTITY_MAP);
        List<Map<String, Object>> list = query.getResultList();

        return list;
    }

    public Map<String, Object> getReportCnt(String qymc, String origin) {
        Map<String, Object> map=new HashMap<String, Object>();
        StringBuffer sql = new StringBuffer();
        // 整车批次
        sql.append(" select max(to_number(批次)) from retrieval.clcp_zk WHERE 企业名称=? ");
        Query query = entityManager.createNativeQuery(sql.toString());
        query.setParameter(1, qymc);
        Long maxpc = Long.parseLong(query.getSingleResult().toString());

        // 如果没有整车批次那就取底盘批次
        if(maxpc==0){
            StringBuffer dpsql = new StringBuffer();
            dpsql.append(" select max(to_number(批次)) from retrieval.chpdpk_zk WHERE 企业名称=? ");
            query = entityManager.createNativeQuery(dpsql.toString());
            query.setParameter(1, qymc);
            maxpc = Long.parseLong(query.getSingleResult().toString());
            map.put("pc", maxpc);
        } else {
            map.put("pc", maxpc);
        }

        // 报告件数 origin （0：检测报告查询界面 | 1：检测报告下载界面）
        StringBuffer cntsql = new StringBuffer();
        cntsql.append(" select count(1) from ggcain06.bgk WHERE qymc=? and confirm='Y'");
        if ("0".equals(origin)) {
            cntsql.append(" and jgdh is null");
        }
        query = entityManager.createNativeQuery(cntsql.toString());
        query.setParameter(1, qymc);
        Long cnt = Long.parseLong(query.getSingleResult().toString());
        map.put("reportCnt", cnt);
        return map;
    }

    public Long checkMbxm(String name) {
        StringBuffer sql = new StringBuffer();
        sql.append(" select count(*) from nerds.ggcs_mb WHERE name=? ");
        Query query = entityManager.createNativeQuery(sql.toString());
        query.setParameter(1, name);
        return Long.parseLong(query.getSingleResult().toString());
    }

    public List<Map<String, Object>> findMbInfoByIds(String mbids) {
        // 默认查公告整车参数
        StringBuilder sql = new StringBuilder("SELECT a.*,case when type='ZC' then 1 when type='DP' then 2 else 3 end  as sortAsc FROM NERDS.GGCS_MB a WHERE ID in ("+mbids+") ORDER BY sortAsc ");
        Query query = entityManager.createNativeQuery(sql.toString());
        query.unwrap(org.hibernate.SQLQuery.class).setResultTransformer(Transformers.ALIAS_TO_ENTITY_MAP);
        //query.setParameter(1, mbids);
        List<Map<String, Object>> list = query.getResultList();

        for(Map<String, Object> data:list){
            for(String key:data.keySet()){
                if(data.get(key) instanceof Clob){
                    Clob clob  = (Clob) data.get(key);
                    try {
                        data.put(key, clob.getSubString((long)1,(int)clob.length()));
                    } catch (SQLException e) {
                        e.printStackTrace();
                    }
                }
            }
        }

        return list;
    }

    public int delete(String mbids) {
        String sql = "DELETE FROM NERDS.GGCS_MB WHERE ID in ("+mbids+") ";
        Query query = entityManager.createNativeQuery(sql.toString());

        return query.executeUpdate();
    }


    @Transactional
    @Modifying
    public void importExcel(List<NerdsInfoEO> result) {

        for (NerdsInfoEO eo : result) {
            String insSql = " INSERT INTO NERDS.ZK_JSCS_2021(ID,PZ_ID,PARAMNAME,PARAMVALUE,QYMC,QY_ID,CPXH,CPMC,CPSB,GZKZLB,XPC,PC,CREATETIME)  VALUES (?, ?, ?, ?, ?, '', ?,?,?,?,?,?, sysdate)";
            Query insQuery = entityManager.createNativeQuery(insSql);
            insQuery.setParameter(1, java.util.UUID.randomUUID().toString().replace("-", "").toLowerCase());
            insQuery.setParameter(2, eo.getPz_id ().trim ());
            insQuery.setParameter(3, eo.getParamName ());
            insQuery.setParameter(4, eo.getParamValue ());
            insQuery.setParameter(5, eo.getQymc ());
            insQuery.setParameter(6, eo.getCpxh ());
            insQuery.setParameter(7, eo.getCpmc ());
            insQuery.setParameter(8, eo.getCpsb ());
            String gzkzlb = "X";
            if(StringUtils.isNotEmpty (eo.getxPc ())){
                gzkzlb="G";
            }
            insQuery.setParameter(9, gzkzlb);
            insQuery.setParameter(10, eo.getxPc ());
            insQuery.setParameter(11, eo.getPc ());
            insQuery.executeUpdate();
        }

        // 更新企业ID
        String sql = "update NERDS.ZK_JSCS_2021 a set qy_id = (SELECT qy_id FROM ggcain06.qyname b WHERE b.name=a.qymc and b.kt='Y' and b.cplb='1' and rownum = 1) where qy_id is null ";
        Query insQuery = entityManager.createNativeQuery(sql);
        insQuery.executeUpdate();

    }

    public List<MbxmEO> queryBaxmList() {

        StringBuilder sql = new StringBuilder("SELECT NAME,SEQ,ZK_NAME FROM NERDS.BACS_MBXM ORDER BY TO_NUMBER(ID) ");
        Query query = entityManager.createNativeQuery(sql.toString());
        query.unwrap(org.hibernate.SQLQuery.class).setResultTransformer(Transformers.ALIAS_TO_ENTITY_MAP);

        List<Map<String, Object>> list = query.getResultList();

        List<MbxmEO> mbxms = new ArrayList<>();
        for (Map<String, Object> resultMap : list) {
            MbxmEO mbxm = new MbxmEO();
            if (null != resultMap.get("NAME")) {
                mbxm.setName (resultMap.get("NAME").toString());
            }
            if (null != resultMap.get("SEQ")) {
                mbxm.setSeq(resultMap.get("SEQ").toString());
            }
            if (null != resultMap.get("ZK_NAME")) {
                mbxm.setZkName(resultMap.get("ZK_NAME").toString());
            }
            mbxms.add(mbxm);
        }
        return mbxms;

    }

    public List<MbxmEO> queryGgxmList() {

        StringBuilder sql = new StringBuilder("SELECT NAME,SEQ,ZK_NAME FROM NERDS.GGCSXM_ALL ORDER BY TYPE, TO_NUMBER(ID) ");
        Query query = entityManager.createNativeQuery(sql.toString());
        query.unwrap(org.hibernate.SQLQuery.class).setResultTransformer(Transformers.ALIAS_TO_ENTITY_MAP);

        List<Map<String, Object>> list = query.getResultList();

        List<MbxmEO> mbxms = new ArrayList<>();
        for (Map<String, Object> resultMap : list) {
            MbxmEO mbxm = new MbxmEO();
            if (null != resultMap.get("NAME")) {
                mbxm.setName (resultMap.get("NAME").toString());
            }
            if (null != resultMap.get("SEQ")) {
                mbxm.setSeq(resultMap.get("SEQ").toString());
            }
            if (null != resultMap.get("ZK_NAME")) {
                mbxm.setZkName(resultMap.get("ZK_NAME").toString());
            }
            mbxms.add(mbxm);
        }
        return mbxms;

    }

    public int deleteByMbmc(String mbmc, String qyid) {
        String sql = "DELETE FROM NERDS.GGCS_MB WHERE name in ("+mbmc+") and qyid = '"+qyid+"'";
        System.out.println (sql);
        Query insQuery = entityManager.createNativeQuery(sql);
        int i = insQuery.executeUpdate();
        return i;
    }
}
