package com.adc.da.bs.param.rest;

import com.adc.da.bs.param.entity.MbxmEO;
import com.adc.da.bs.param.entity.MbxmVO;
import com.adc.da.bs.param.service.MbService;
import com.adc.da.util.exception.AdcDaBaseException;
import com.adc.da.util.http.PageParam;
import com.adc.da.util.http.ResponseMessage;
import com.adc.da.util.http.Result;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.util.List;
import java.util.Map;

@RestController
@RequestMapping("/${restPath}/mb/")
@Api(tags = "mb", description = "模板相关功能")
public class MbController {

    private static Logger logger = LoggerFactory.getLogger(MbController.class);


    @Autowired
    private MbService mbService;

    @PostMapping("queryMbxmList")
    @ApiOperation("模板项目列表")
    public ResponseMessage<List<MbxmEO>> queryMbxmList(String type) {
        return Result.success(mbService.queryMbxmList(type));
    }

    /**
     * 单个
     *
     * @param mbxmVO
     * @return
     */
    @PostMapping("createMbxm")
    @ApiOperation("模板项目创建")
    public ResponseMessage createMbxm(HttpServletRequest request, @RequestBody MbxmVO mbxmVO) {
        try {

            Map<String, Object> usermap = (Map<String, Object>) request.getSession().getAttribute("usermap");
            String qyid = usermap.get("USERNAME").toString();
            logger.info("******USERNAME" + qyid);

            mbxmVO.setQyid(qyid);
            mbService.createMbxm(mbxmVO);
            return Result.success("创建模板成功");
        } catch (AdcDaBaseException e) {
            return Result.error(e.getMessage());
        }

    }

    /**
     * 模板项目信息列表（按企业查询）
     * zhong
     * 2021-1-5
     */
    @ApiOperation(value = "查询本企业的模板项目信息")
    @PostMapping("/findmbListbyQyid")
    @ResponseBody
    public Map<String, Object> findMbListByQyid(HttpServletRequest request, PageParam page, String mbmc) {
        Map<String, Object> usermap = (Map<String, Object>) request.getSession().getAttribute("usermap");
        String qyid = usermap.get("USERNAME").toString();

        logger.info("******USERNAME" + qyid);
        return mbService.findMbListByQyid(qyid, mbmc);
    }

    /**
     * 按模板导出表格数据
     */
    @ApiOperation(value = "按模板导出表格数据", produces = "application/octet-stream")
    @RequestMapping(value = "/export", method = RequestMethod.POST)
    public ResponseMessage exportExcel(HttpServletRequest request, @RequestBody Map<String, Object> map, String mbname, HttpServletResponse response) throws IOException {

        logger.info("按模板导出表格数据:{exportExcel}" + map.get("selectedrows"));
        // mbid 传输的是模板名称，因为公告备案可以一起导mbid就不唯一了
        logger.info("测试:{mbname}" + mbname);
        Map<String, Object> usermap = (Map<String, Object>) request.getSession().getAttribute("usermap");
        String qyid = usermap.get("USERNAME").toString();

        logger.info("******USERNAME" + qyid);
        long start = System.currentTimeMillis();

        StringBuffer mbid = new StringBuffer();
        /*for (int i=0;i<mbids.length;i++) {
            if(i==0){
                mbid.append("'").append(mbids[i]).append("'");
            }else{
                mbid.append(",").append("'").append(mbids[i]).append("'");
            }
        }*/
        try {
            //mbService.exportExcel(map, name, response);
            boolean checkData = false;
            mbService.exportExcelNew(map, qyid, mbname, response, checkData);
        } catch (AdcDaBaseException e) {
            return Result.error(e.getMessage());
        } catch (Exception e) {
            return Result.error(e.getMessage());
        }
        logger.info("导出excel所花时间：" + (System.currentTimeMillis() - start));
        return Result.success();
    }

    /**
     * 按模板导出表格数据
     */
    @ApiOperation(value = "校验按模板导出表格数据", produces = "application/octet-stream")
    @RequestMapping(value = "/checkExport", method = RequestMethod.POST)
    public ResponseMessage checkExportExcel(HttpServletRequest request, @RequestBody Map<String, Object> map, String mbname, HttpServletResponse response) throws IOException {

        logger.info("校验按模板导出表格数据:{checkExportExcel}" + map.get("selectedrows"));
        logger.info("测试:{mbname}" + mbname);
        Map<String, Object> usermap = (Map<String, Object>) request.getSession().getAttribute("usermap");
        String qyid = usermap.get("USERNAME").toString();

        logger.info("******USERNAME" + qyid);
        long start = System.currentTimeMillis();
        StringBuffer mbid = new StringBuffer();

        try {
            boolean checkData = true;
            mbService.exportExcelNew(map, qyid, mbname, response, checkData);
        } catch (AdcDaBaseException e) {
            return Result.error(e.getMessage());
        } catch (Exception e) {
            return Result.error(e.getMessage());
        }
        logger.info("导出excel所花时间：" + (System.currentTimeMillis() - start));
        return Result.success();
    }

    /**
     * 申报数据-参数对比
     */
    @ApiOperation(value = "申报数据-参数对比")
    @PostMapping("/findListbyParams")
    @ResponseBody
    public Map<String, Object> findListbyParams(@RequestBody Map<String, Object> map) {
        logger.info("测试:{findListbyParams}" + map.get("selectedrows"));

        Map<String,Object> pageMap =mbService.findListbyParams(map);

        return pageMap;
    }

    /**
     * 按参数导出表格数据
     */
    @ApiOperation(value = "按参数导出表格数据", produces = "application/octet-stream")
    @RequestMapping(value = "/exportParams", method = RequestMethod.POST)
    public void exportParamsExcel(@RequestBody Map<String, Object> map, String mbid, HttpServletResponse response, String type) throws IOException {

        logger.info("测试:{exportParams}" + map.get("selectedrows"));
        logger.info("测试:{mbid}" + mbid);
        logger.info("测试:{type}" + type);

        long start = System.currentTimeMillis();

        Map<String,Object> pageMap =mbService.findListbyParams(map);
//        mbService.exportParamsExcel(map, response);
        mbService.exportParamsExcelNew(pageMap, type, response);
        logger.info("导出excel所花时间：" + (System.currentTimeMillis() - start));

    }

    /**
     * 申报数据-参数对比
     */
    @ApiOperation(value = "检测报告数量")
    @PostMapping("/getReportCnt")
    @ResponseBody
    public Map<String, Object> getReportCnt(HttpServletRequest request, String origin) {
        Map<String, Object> usermap = (Map<String, Object>) request.getSession().getAttribute("usermap");
        // 获取企业名称
        String qymc=usermap.get("NAME").toString();
        logger.info("******qymc" + qymc);
        Map<String,Object> pageMap =mbService.getReportCnt(qymc, origin);

        return pageMap;
    }

    /**
     * 下载备案参数
     */
    @ApiOperation(value = "下载备案参数", produces = "application/octet-stream")
    @RequestMapping(value = "/exportBaParamsExcel", method = RequestMethod.POST)
    public ResponseMessage exportBaParamsExcel(@RequestBody Map<String, Object> map, HttpServletResponse response) throws IOException {

        logger.info("测试:{exportBaParamsExcel}" + map.get("selectedrows"));

        long start = System.currentTimeMillis();
        String mbid = "1234567890BA";
        try {
            boolean checkData = false;
            mbService.exportExcel(map, mbid, response, checkData);
        } catch (AdcDaBaseException e) {
            if(e.getMessage().contains("上传公告")) {
                return Result.error("您选择的参数未在模板中，请确认是否已上传公告有效参数表！");
            } else if(e.getMessage().contains("模板信息")){
                return Result.error("该模板信息不存在.");
            }
        } catch (Exception e) {
            return Result.error(e.getMessage());
        }
        logger.info("导出excel所花时间：" + (System.currentTimeMillis() - start));
        return Result.success();
    }

    /**
     * 下载备案参数
     */
    @ApiOperation(value = "校验下载备案参数", produces = "application/octet-stream")
    @RequestMapping(value = "/checkExportBaParamsExcel", method = RequestMethod.POST)
    public ResponseMessage checkExportBaParamsExcel(@RequestBody Map<String, Object> map, HttpServletResponse response) throws IOException {

        logger.info("测试:{checkExportBaParamsExcel}" + map.get("selectedrows"));

        long start = System.currentTimeMillis();
        String mbid = "1234567890BA";
        try {
            boolean checkData = true;
            mbService.exportExcel(map, mbid, response, checkData);
        } catch (AdcDaBaseException e) {
            return Result.error(e.getMessage());
        } catch (Exception e) {
            return Result.error(e.getMessage());
        }
        logger.info("导出excel所花时间：" + (System.currentTimeMillis() - start));
        return Result.success();
    }

    /**
     * @describtion 删除
     * @author zhong
     * @modified by
     */
    @ApiOperation(value = "删除模板")
    @PostMapping("/delete")
    @ResponseBody
    public ResponseMessage delete(HttpServletRequest request,  String[] mbmcs ) {
        StringBuffer mbmc = new StringBuffer();
        for (int i=0;i<mbmcs.length;i++) {
            if(i==0){
                mbmc.append("'").append(mbmcs[i]).append("'");
            }else{
                mbmc.append(",").append("'").append(mbmcs[i]).append("'");
            }
        }
        // 获取企业名称
        Map<String, Object> usermap = (Map<String, Object>) request.getSession().getAttribute("usermap");
        String qyid = usermap.get("USERNAME").toString();

        logger.info("******qyid" + qyid);
        int n = mbService.deleteByMbmc (mbmc.toString (), qyid);
        if (0 != n) {
            return Result.success();
        } else {
            return Result.error("模板删除失败.");
        }
    }

    @PostMapping("/showBacsXmList")
    @ApiOperation("预览备案参数")
    public ResponseMessage<List<MbxmEO>> showBacsXmList(@RequestBody Map<String, Object> map, HttpServletResponse response) {
        logger.info("测试:{showBacsXmList}" + map.get("selectedrows"));

        long start = System.currentTimeMillis();
        String mbid = "1234567890BA";
        try {
            return Result.success(mbService.showBacsXmList(map, mbid));
        } catch (AdcDaBaseException e) {
            return Result.error(e.getMessage());
        } catch (Exception e) {
            return Result.error(e.getMessage());
        }

    }


    /**
     * 校验下载公告参数
     */
    @ApiOperation(value = "校验下载公告参数")
    @RequestMapping(value = "/checkExportGgParamsExcel", method = RequestMethod.POST)
    public ResponseMessage checkExportGgParamsExcel(@RequestBody Map<String, Object> map, HttpServletResponse response) throws IOException {

        logger.info("测试:{checkExportGgParamsExcel}" + map.get("selectedrows"));

        long start = System.currentTimeMillis();
        try {
            boolean checkData = true;
            mbService.checkExportGgParamsExcel(map, response, checkData);
        } catch (AdcDaBaseException e) {
            return Result.error(e.getMessage());
        } catch (Exception e) {
            return Result.error(e.getMessage());
        }
        logger.info("测试:{checkExportGgParamsExcel} 所花时间：" + (System.currentTimeMillis() - start));
        return Result.success();
    }

    /**
     * 下载公告参数
     */
    @ApiOperation(value = "下载公告参数")
    @RequestMapping(value = "/exportGgParamsExcel", method = RequestMethod.POST)
    public void exportGgParamsExcel(@RequestBody Map<String, Object> map, HttpServletResponse response) throws IOException {

        logger.info("测试:{exportGgParamsExcel}" + map.get("selectedrows"));

        long start = System.currentTimeMillis();
        try {
            boolean checkData = false;
            mbService.exportGgParamsExcel(map, response);
        } catch (Exception e) {
            e.printStackTrace();
        }
        logger.info("测试:{exportGgParamsExcel} 所花时间：" + (System.currentTimeMillis() - start));
    }
}
