package com.adc.da.bs.param.rest;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.TimeUnit;

import javax.servlet.http.HttpServletRequest;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.bind.annotation.RestController;

import com.adc.da.bs.login.service.LoginService;
import com.adc.da.bs.param.service.ParamChartsService;
import com.adc.da.util.utils.RedisUtils;

/**
 * 车型参数Controller
 * ADC_ZH
 * 2021-1-14
 */

@RestController
@RequestMapping(value = "/${restPath}/paramCharts")
@Api(description = "公告数据分析")
public class ParamChartsController {

	@Autowired
	private ParamChartsService paramChartsService;
	@Autowired
	private LoginService loginService;

    @Autowired
    private RedisUtils redisUtils;
	/**
	 * 折线图
	 * ADC_ZH
	 * 2021-1-15
	 */
	@ApiOperation(value = "折线图-近两年数据比对")
	@PostMapping("/lineCharts")
	@ResponseBody
	public Map<String, Object> lineCharts(HttpServletRequest request) {
 		Map<String,Object> pageMap =new HashMap();
		Map<String,Object> usermap=(Map<String, Object>) request.getSession().getAttribute("usermap");
		if(usermap==null||usermap.isEmpty()){
			pageMap.put("msg", "未登录");
			pageMap.put("result", "-1");
			return pageMap;
		}
		String qyid=usermap.get("USERNAME").toString();
		pageMap=paramChartsService.lineCharts(qyid);
		return pageMap;
 	}
	
	/**
	 * 排行榜
	 * ADC_ZH
	 * 2021-1-18
	 */
	@ApiOperation(value = "车型、品牌-排行榜top10")
	@PostMapping("/barCharts")
	@ResponseBody
	public Map<String, Object> barCharts(HttpServletRequest request) {
 		Map<String,Object> pageMap =new HashMap();
		Map<String,Object> usermap=(Map<String, Object>) request.getSession().getAttribute("usermap");
		if(usermap==null||usermap.isEmpty()){
			pageMap.put("msg", "未登录");
			pageMap.put("result", "-1");
			return pageMap;
		}
		pageMap=paramChartsService.barCharts();
		return pageMap;
 	}
	
	/**
	 * 排行榜
	 * ADC_ZH
	 * 2021-1-18
	 */
	@ApiOperation(value = "同比-环比")
	@PostMapping("/compare")
	@ResponseBody
	public Map<String, Object> compare(HttpServletRequest request) {
 		Map<String,Object> pageMap =new HashMap();
		Map<String,Object> usermap=(Map<String, Object>) request.getSession().getAttribute("usermap");
		if(usermap==null||usermap.isEmpty()){
			pageMap.put("msg", "未登录");
			pageMap.put("result", "-1");
			return pageMap;
		}
		String qyid=usermap.get("USERNAME").toString();
		if(!redisUtils.exists("compare_"+qyid)){
			long min=60*24*7; 
			pageMap=paramChartsService.compare(qyid);
            redisUtils.set("compare_"+qyid, pageMap, min, TimeUnit.MINUTES);
		}else{
			pageMap=(Map<String, Object>) redisUtils.get("compare_"+qyid);
		}
		return pageMap;
 	}
	
}
