package com.adc.da.bs.policy.dao;

import com.adc.da.util.exception.AdcDaBaseException;
import com.adc.da.util.http.PageParam;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.hibernate.transform.Transformers;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.jpa.repository.Modifying;
import org.springframework.stereotype.Component;

import javax.persistence.EntityManager;
import javax.persistence.Query;
import javax.transaction.Transactional;
import java.util.List;
import java.util.Map;

/**
 * @author 韩磊
 * @date 2019/3/21 13:42
 */
@Slf4j
@Component
public class PolicyDao {
    protected final EntityManager entityManager;

    @Autowired
    public PolicyDao(EntityManager entityManager) {
        super();
        this.entityManager = entityManager;
    }

    public List<Map<String, Object>> policyList(String property) {
        StringBuilder sql = new StringBuilder();
        sql.append("  SELECT ROWNUM,TAFM.FILE_ID,TAFM.TITLE ,TAFM.AUTHOR,TAFM.PAGE_NUM,TAFM.CHARTS_NUM,");
        sql.append(" to_char(TAFM.UPDATE_DATE ,'yyyy-MM-DD') as UPDATE_DATE  ,TAFM.UPDATE_ID,TAFM.FURL,TAFM.FTYPE,TAFM.PIC_ID,TAFM.CONTENT,TAFM.TOP,TAPM.PURL ");
        sql.append(" FROM NERDS.TS_ADCBS_FILE_MANAGERMENT  TAFM");
        sql.append(" left join NERDS.TS_ADCBS_PICTURE_MANAGERMENT  TAPM");
        sql.append(" on TAFM.PIC_ID=TAPM.PIC_ID  ");
        sql.append(" WHERE TAFM.ENABLE='0'  AND TAFM.FPROPERTY = ? ");
        sql.append("   ORDER BY TAFM.UPDATE_DATE DESC  ");
        Query query = entityManager.createNativeQuery(sql.toString());
        query.unwrap(org.hibernate.SQLQuery.class).setResultTransformer(Transformers.ALIAS_TO_ENTITY_MAP);
        query.setParameter(1, property);
        List<Map<String, Object>> list = query.getResultList();
        return list;
    }


    public int uploadFile(Map<String, Object> map) {
        //获取pic的Id
        String idSql = "SELECT NERDS.retrieval_seq.nextval from dual";
        Query idQuery = entityManager.createNativeQuery(idSql);
        Integer picId = Integer.parseInt(idQuery.getSingleResult().toString());
        String inPicSql =
                " INSERT INTO NERDS.TS_ADCBS_PICTURE_MANAGERMENT (PIC_ID,PNAME, PURL, PTYPE ,PSIZE, PSUFFIX, " +
                        "PPROPERTY, ENABLE, CREATE_DATE,CREATE_ID,UPDATE_DATE,UPDATE_ID )" +
                        " VALUES (?, ?, ?, ?, ?, ?, ? , ?, SYSDATE, ?, SYSDATE ,?) ";
        Query inPicQuery = entityManager.createNativeQuery(inPicSql);
        inPicQuery.setParameter(1, picId);
        inPicQuery.setParameter(2, map.get("pname"));
        inPicQuery.setParameter(3, map.get("purl"));
        inPicQuery.setParameter(4, map.get("ptype"));
        inPicQuery.setParameter(5, map.get("psize"));
        inPicQuery.setParameter(6, map.get("psuffix"));
        inPicQuery.setParameter(7, map.get("pproperty"));
        inPicQuery.setParameter(8, 0);
        inPicQuery.setParameter(9, map.get("userId"));
        inPicQuery.setParameter(10, map.get("userId"));

        int i = 0;
        try {
            i = inPicQuery.executeUpdate();
        } catch (Exception e) {
            log.error(e.toString());
            throw new AdcDaBaseException("封面图片存储失败：" + map.get("pname"), e.toString());
        }

        String inFileSql =
                " INSERT INTO NERDS.TS_ADCBS_FILE_MANAGERMENT (FILE_ID,TITLE, PIC_ID, AUTHOR ,PAGE_NUM, " +
                        "CHARTS_NUM, FNAME, FURL, FSIZE,CONTENT,TOP,FTYPE,FSUFFIX, FPROPERTY,ENABLE," +
                        "CREATE_DATE,CREATE_ID,UPDATE_DATE,UPDATE_ID)" +
                        " VALUES (NERDS.retrieval_seq.nextval, ?,?,?,?,?,?,?,?,?,?,?,?,?,?, SYSDATE, ?, SYSDATE ,?) ";
        Query inFileQuery = entityManager.createNativeQuery(inFileSql);
        inFileQuery.setParameter(1, map.get("title"));
        inFileQuery.setParameter(2, picId);
        inFileQuery.setParameter(3, map.get("author"));
        inFileQuery.setParameter(4, map.get("page"));
        inFileQuery.setParameter(5, map.get("charts"));
        inFileQuery.setParameter(6, map.get("fname"));
        inFileQuery.setParameter(7, map.get("furl"));
        inFileQuery.setParameter(8, map.get("fsize"));
        inFileQuery.setParameter(9, map.get("content"));
        inFileQuery.setParameter(10, map.get("top"));
        inFileQuery.setParameter(11, map.get("ftype"));
        inFileQuery.setParameter(12, map.get("fsuffix"));
        inFileQuery.setParameter(13, map.get("fproperty"));
        inFileQuery.setParameter(14, 0);
        inFileQuery.setParameter(15, map.get("userId"));
        inFileQuery.setParameter(16, map.get("userId"));

        i = 0;
        try {
            i = inFileQuery.executeUpdate();
        } catch (Exception e) {
            log.error(e.toString());
            throw new AdcDaBaseException("文件存储插入失败：" + map.get("fname"));
        }

        return i;
    }

    /**
     * @param @param    page
     * @param @param    type
     * @param @param    keyName
     * @param @return   设定文件
     * @param fPROPERTY
     * @return List<Map                                                               <                                                               String                                                               ,                                                               Object>>    返回类型
     * @Title: findListByPage
     * @Description: TODO(这里用一句话描述这个方法的作用)
     * @author ADC_ZH
     * @date 上午10:53:47
     */
    public List<Map<String, Object>> findListByPage(PageParam page,
                                                    String type, String keyName, String fproperty) {
        StringBuffer sql = new StringBuffer(" SELECT ROWNUM RN ,T.* FROM ( ");
        sql.append("  SELECT ROWNUM,TAFM.FILE_ID,TAFM.TITLE ,TAFM.AUTHOR,TAFM.PAGE_NUM,TAFM.CHARTS_NUM,");
        sql.append("  to_char(TAFM.UPDATE_DATE ,'yyyy-MM-DD') as UPDATE_DATE ,TAFM.UPDATE_ID,TAFM.FURL,TAFM.FTYPE,TAFM.PIC_ID,TAFM.CONTENT,TAFM.TOP,TAPM.PURL,TAFM.FPROPERTY ");
        sql.append(" FROM NERDS.TS_ADCBS_FILE_MANAGERMENT  TAFM");
        sql.append(" left join NERDS.TS_ADCBS_PICTURE_MANAGERMENT  TAPM");
        sql.append(" on TAFM.PIC_ID=TAPM.PIC_ID  ");
        sql.append(" WHERE TAFM.ENABLE='0' ");
        if (StringUtils.isNotEmpty(fproperty)) {    //判断是否输入类别的关键字
            sql.append("AND TAFM.FPROPERTY = '" + fproperty + "'");
        }
        if (StringUtils.isNotEmpty(keyName)) {    //判断是否输入查询的关键字
            sql.append("AND TAFM.TITLE LIKE '%" + keyName + "%'");
        }
        if (StringUtils.isNotEmpty(type)) {    //判断是否输入查询的关键字
            sql.append("AND TAFM.AUTHOR LIKE '%" + type + "%'");
        }
        if (StringUtils.isNotEmpty(page.getStartTime())) {            //判断是否输入起始时间
            sql.append("AND to_char(TAFM.UPDATE_DATE,'yyyy-MM-dd HH24:mi:ss')>='" + page.getStartTime() + "'");
        }
        if (StringUtils.isNotEmpty(page.getEndTime())) {            //判断是否输入结束时间
            sql.append("AND to_char(TAFM.UPDATE_DATE,'yyyy-MM-dd HH24:mi:ss')<='" + page.getEndTime() + "'");
        }
        sql.append("ORDER BY TAFM.UPDATE_DATE DESC");
        sql.append(" ) T ");
        Query query = entityManager.createNativeQuery(sql.toString());
        query.unwrap(org.hibernate.SQLQuery.class).setResultTransformer(Transformers.ALIAS_TO_ENTITY_MAP);
        query.setFirstResult(page.getOffset());
        query.setMaxResults(page.getLimit());
        List<Map<String, Object>> list = query.getResultList();
        return list;
    }

    /**
     * @param @param    page
     * @param @param    type
     * @param @param    keyName
     * @param @return   设定文件
     * @param fPROPERTY
     * @return Long    返回类型
     * @Title: findTotalCount
     * @Description: TODO(这里用一句话描述这个方法的作用)
     * @author ADC_ZH
     * @date 上午10:53:52
     */
    public Long findTotalCount(PageParam page, String type, String keyName, String fproperty) {
        StringBuilder sql = new StringBuilder();
        sql.append(" SELECT count(*) from NERDS.TS_ADCBS_FILE_MANAGERMENT WHERE ENABLE='0' ");
        if (StringUtils.isNotEmpty(fproperty)) {    //判断是否输入查询的关键字
            sql.append("AND FPROPERTY = '" + fproperty + "'");
        }
        if (StringUtils.isNotEmpty(keyName)) {    //判断是否输入查询的关键字
            sql.append("AND TITLE LIKE '%" + keyName + "%'");
        }
        if (StringUtils.isNotEmpty(type)) {    //判断是否输入查询的关键字
            sql.append("AND AUTHOR LIKE '%" + type + "%'");
        }
        Query query = entityManager.createNativeQuery(sql.toString());
        Long count = Long.parseLong(query.getSingleResult().toString());
        return count;
    }

    /**
     * @param @param  id
     * @param @return 设定文件
     * @return boolean    返回类型
     * @Title: delete
     * @Description: TODO(这里用一句话描述这个方法的作用)
     * @author ADC_ZH
     * @date 下午6:00:30
     */
    @Transactional
    @Modifying
    public boolean delete(String id) {
        boolean re = false;
        if (id != null && !"".equals(id)) {
            StringBuffer deleteSql = new StringBuffer();
            deleteSql.append(" UPDATE NERDS.TS_ADCBS_FILE_MANAGERMENT SET ENABLE = '1' ");
            deleteSql.append(" WHERE FILE_ID = '" + id + "'");
            Query query = entityManager.createNativeQuery(deleteSql.toString());
            if (query.executeUpdate() == 1) {
                re = true;
            }
        }
        return re;
    }


}
