package com.adc.da.bs.report.service;

import com.adc.da.bs.report.dao.AnalyzeReportDao;
import com.adc.da.communal.file.store.IFileStore;
import com.adc.da.communal.file.vo.FileUploadQueryVO;
import com.adc.da.util.utils.FileUtil;
import com.adc.da.util.utils.IOUtils;
import com.adc.da.util.utils.StringUtils;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;

import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 分析报告service
 *
 * @author 韩磊
 * @date 2019/3/21 13:41
 */
@Slf4j
@Service
public class AnalyzeReportService {
    @Autowired
    private AnalyzeReportDao analyzeReportDao;

    @Autowired
    private IFileStore iFileStore;

    //默认路径
    @Value("${file.path}")
    private String basePath;

    @Transactional(rollbackFor = Exception.class)
    public int uploadFile(MultipartFile file, MultipartFile pic, FileUploadQueryVO fileUploadQueryVO,
            Map<String, Object> usermap) {
        Map<String, Object> map = new HashMap<String, Object>();

        String filePath = null;
        InputStream in = null;
        InputStream inPic = null;
        String fileName;
        String fileExtension;
        String path;
        int i = 0;
        StringBuilder pathHeader = new StringBuilder();
        pathHeader.append(File.separator);
        pathHeader.append("ADC");
        pathHeader.append(File.separator);
        pathHeader.append("upload");
        pathHeader.append(File.separator);
        Integer picId = null;

        map.put("fproperty", fileUploadQueryVO.getType());
        map.put("title", fileUploadQueryVO.getTitle());
        map.put("page", fileUploadQueryVO.getPageNum());
        map.put("charts", fileUploadQueryVO.getChartsNum());
        map.put("author", fileUploadQueryVO.getAuthor());
        map.put("top", fileUploadQueryVO.getTop());
        map.put("content", fileUploadQueryVO.getContent());
        map.put("userId", usermap.get("ID"));
        map.put("uploadDate", fileUploadQueryVO.getUploadDate());
        try {
            if (null != pic && StringUtils.isNoneBlank(pic.getOriginalFilename())) {
                String picName = pic.getOriginalFilename();
                String picExtension = FileUtil.getFileExtension(picName);
                inPic = pic.getInputStream();
                String pathPic = iFileStore.storeFile(inPic, picExtension, filePath, basePath);
                String purl = pathHeader.toString() + pathPic;
                map.put("pname", FileUtil.getFileName(pic.getOriginalFilename()));
                map.put("purl", purl.replaceAll("\\\\","/"));
                map.put("ptype", picName.substring(picName.lastIndexOf(".") + 1));
                map.put("psize", pic.getSize());
                map.put("psuffix", picName.substring(picName.lastIndexOf(".")));
                //封面
                map.put("pproperty", "cover");
                picId = analyzeReportDao.uploadPic(map);
            }
            if (null != file && StringUtils.isNoneBlank(file.getOriginalFilename())) {
                fileName = file.getOriginalFilename();
                fileExtension = FileUtil.getFileExtension(fileName);
                in = file.getInputStream();
                path = iFileStore.storeFile(in, fileExtension, filePath, basePath);
                String furl = pathHeader.toString() + path;
                map.put("furl", furl.replaceAll("\\\\","/"));
                map.put("ftype", fileName.substring(fileName.lastIndexOf(".") + 1));
                map.put("fsuffix", fileName.substring(fileName.lastIndexOf(".")));
                map.put("fsize", file.getSize());
                map.put("fname", FileUtil.getFileName(file.getOriginalFilename()));
                i = analyzeReportDao.uploadFile(map, picId);
            }
            return i;
        } catch (IOException e) {
            log.error(e.getMessage(), e);
            return 0;
        } finally {
            IOUtils.closeQuietly(in);
        }
    }

    public Map<String, Object> analyzeReportIndex() {
        Map<String, Object> map = new HashMap<>();
        //公告车型分析
        List<Map<String, Object>> list = analyzeReportDao.analyzeReportIndex("1", 5);
        map.put("rows", list);
        return map;
    }

    public Map<String, Object> analyzeNewEnergyIndex() {
        Map<String, Object> map = new HashMap<>();
        //新能源车型分析
        List<Map<String, Object>> list = analyzeReportDao.analyzeReportIndex("2", 5);
        map.put("rows", list);
        return map;
    }

    public Map<String, Object> analyzeReportList() {
        Map<String, Object> map = new HashMap<>();
        //公告车型分析
        List<Map<String, Object>> list = analyzeReportDao.analyzeReportList("1");
        map.put("rows", list);
        return map;
    }

    public Map<String, Object> analyzeNewEnergyList() {
        Map<String, Object> map = new HashMap<>();
        //新能源车型分析
        List<Map<String, Object>> list = analyzeReportDao.analyzeReportList("2");
        map.put("rows", list);
        return map;
    }

}
