package com.adc.da.communal.file.rest;

import com.adc.da.bs.resourceSharing.service.QandAService;
import com.adc.da.communal.file.Entity.FileEO;
import com.adc.da.communal.file.store.IFileStore;
import com.adc.da.util.exception.AdcDaBaseException;
import com.adc.da.util.http.ResponseMessage;
import com.adc.da.util.http.Result;
import com.adc.da.util.utils.Encodes;
import com.adc.da.util.utils.IOUtils;
import com.adc.da.util.utils.StringUtils;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.validation.constraints.NotNull;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import java.util.Map;

@RestController
@RequestMapping(value = "/${restPath}/sys/file")
@Api(description = "test文件下载")
public class FileDownloadRestController {

	private static final Logger logger = LoggerFactory.getLogger(FileDownloadRestController.class);
	@Value("${file.rsUploadPath}")
	private String rsUploadPath;//资源共享上传路径
	@Autowired
	private IFileStore iFileStore;
    @Autowired
    private QandAService qandAService;

	@ApiOperation(value = "|FileEO|下载")
	@GetMapping("/{id}/{type}/download")
	public void downFile(HttpServletRequest request, HttpServletResponse response,@NotNull @PathVariable String id, @NotNull @PathVariable String type) {
        Map<String,Object> res= qandAService.findClassicQFileInfo(id, type);

        if(res.get("FILENAME") == null || res.get("FILEURL") == null){
            throw new AdcDaBaseException("文件不存在");
        }
	    String fileName=res.get("FILENAME").toString();
		String fileURL=res.get("FILEURL").toString();
	    if (StringUtils.isEmpty(fileName) || StringUtils.isEmpty(fileURL)) {
			throw new AdcDaBaseException("文件不存在！");
		}
        try {
            fileName =  URLDecoder.decode(fileName,"UTF-8");
            fileURL =  URLDecoder.decode(fileURL,"UTF-8");
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        }

		String fileType = fileName.substring(fileName.indexOf("."));

		InputStream is = null;
		OutputStream os = null;
		try {

			response.setHeader("Content-Disposition", "attachment; filename=" + Encodes.urlEncode(fileName));
			response.setContentType(fileType);
			is = iFileStore.loadFile(fileURL, rsUploadPath);
			os = response.getOutputStream();
			IOUtils.copy(is, os);
			os.flush();
		} catch (IOException e) {
			logger.error(e.getMessage(), e);
			throw new AdcDaBaseException("下载文件失败，请重试");
		} finally {
			IOUtils.closeQuietly(is);
			IOUtils.closeQuietly(os);
		}
	}
	
	
	@ApiOperation(value = "|FileEO|下载-2021")
	@GetMapping("/downloadFiles")
	public void downFiles(HttpServletRequest request, HttpServletResponse response, String fileName,  String fileURL) {

	    if ( StringUtils.isEmpty(fileURL)) {
			throw new AdcDaBaseException("文件不存在！");
		}
        try {
//            fileName =  URLDecoder.decode(fileName,"UTF-8");
            fileURL =  URLDecoder.decode(fileURL,"UTF-8");
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        }

//		String fileType = fileName.substring(fileName.indexOf("."));

		InputStream is = null;
		OutputStream os = null;
		try {

//			response.setHeader("Content-Disposition", "attachment; filename=" + Encodes.urlEncode(fileName));
//			response.setContentType(fileType);
			is = iFileStore.loadFile(fileURL, rsUploadPath);
			os = response.getOutputStream();
			IOUtils.copy(is, os);
			os.flush();
		} catch (IOException e) {
			logger.error(e.getMessage(), e);
			throw new AdcDaBaseException("下载文件失败，请重试");
		} finally {
			IOUtils.closeQuietly(is);
			IOUtils.closeQuietly(os);
		}
	}
}
