package com.adc.da.communal.file.rest;

import com.adc.da.bs.report.enums.FileTypeEnum;
import com.adc.da.bs.report.service.AnalyzeReportService;
import com.adc.da.communal.file.vo.FileUploadQueryVO;
import com.adc.da.util.http.ResponseMessage;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.annotation.PostConstruct;
import javax.servlet.http.HttpServletRequest;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;

import com.adc.da.bs.report.service.ReportService;
import com.adc.da.communal.file.Entity.FileEO;
import com.adc.da.communal.file.store.IFileStore;
import com.adc.da.util.http.Result;
import com.adc.da.util.utils.FileUtil;
import com.adc.da.util.utils.IOUtils;
import com.adc.da.util.utils.StringUtils;

@RestController
@RequestMapping(value = "/${restPath}/sys/file")
@Api(description = "test文件上传")
public class FileUploadRestController {

    private static final Logger logger = LoggerFactory.getLogger(FileUploadRestController.class);
    @Value("${file.path}")
    private String basePath;//默认路径
    @Value("${file.rsUploadPath}")
    private String rsUploadPath;//资源共享上传路径
    @Autowired
    private IFileStore iFileStore;
    @Autowired
    private ReportService reportService;

    @Autowired
    private AnalyzeReportService analyzeReportService;

    // 上传文件类型黑名单
    private List<String> blackUrls;


    @PostConstruct
    public void init() {
        // 读取文件
        String path = FileUploadRestController.class.getResource("/").getFile();
        blackUrls = FileUtil.readAsStringList(path + "uploadBlack.txt");
    }

    /**
     * 文件上传
     * @param file 指定文件上传路径，空则在根路径
     * @return
     */
    @ApiOperation(value = "|FileEO|上传")
    @PostMapping("/upload")
    public ResponseMessage<FileEO> upload(HttpServletRequest request, @RequestParam("file") MultipartFile file) {

        String filePath = null;

        FileEO fileEO;
        InputStream is = null;
        try {
            String fileExtension = FileUtil.getFileExtension(file.getOriginalFilename());

            if (blackUrls.contains(fileExtension)) {
                logger.error("上传文件类型不允许，请重试");
                return Result.error("r0071", "上传文件类型不允许，请重试");
            }

            is = file.getInputStream();
            String path = iFileStore.storeFile(is, fileExtension, filePath, rsUploadPath);

            fileEO = new FileEO();
            fileEO.setFileName(FileUtil.getFileName(file.getOriginalFilename()));
            fileEO.setFileType(fileExtension);
            fileEO.setContentType(file.getContentType());
            fileEO.setSavePath(path);
            fileEO.setCreateTime(new Date());
        } catch (IOException e) {
            logger.error(e.getMessage(), e);
            return Result.error("r0072", "文件存储失败，请重试");
        } finally {
            IOUtils.closeQuietly(is);
        }
        return Result.success(fileEO);
    }

    /**
     * 多文件上传
     * @param files 指定文件上传路径，空则在根路径
     * @return
     */
    @ApiOperation(value = "|FileEO|上传")
    @PostMapping("/uploads")
    public ResponseMessage<List<FileEO>> uploads( @RequestParam("file") MultipartFile[] files,String id) {

        String filePath = null;
        String uploadPath = null;

        List<FileEO> fileEOs = new ArrayList<FileEO>();
        InputStream is = null;
        try {
            for (MultipartFile file : files) {
                String fileExtension = FileUtil.getFileExtension(file.getOriginalFilename());

                if (blackUrls.contains(fileExtension)) {
                    logger.error("上传文件类型不允许，请重试");
                    return Result.error("r0071", "上传文件类型不允许，请重试");
                }

                is = file.getInputStream();
                String path = iFileStore.storeFile(is, fileExtension, filePath,uploadPath);

                FileEO fileEO = new FileEO();
                fileEO.setAppId(id);
                fileEO.setFileName(StringUtils.deleteWhitespace(FileUtil.getFileName(file.getOriginalFilename())));
                fileEO.setFileType(fileExtension);
                fileEO.setContentType(file.getContentType());
                fileEO.setSavePath(path);
                fileEO.setCreateTime(new Date());
                fileEO.setDelFlag(0);
                fileEOs.add(fileEO);
            }
        } catch (IOException e) {
            logger.error(e.getMessage(), e);
            return Result.error("r0072", "文件存储失败，请重试");
        } finally {
            IOUtils.closeQuietly(is);
        }
        return Result.success(fileEOs);
    }


    /**
     * 文件上传
     * @param file 指定文件上传路径，空则在根路径
     * @return
     */
    @SuppressWarnings("unchecked")
    @ApiOperation(value = "|FileEO|上传")
    @PostMapping("/uploadPDF")
    public Map<String, Object> uploadPDF(HttpServletRequest request, @RequestParam("file") MultipartFile file,
            @RequestParam("filepic") MultipartFile filepic,String title
            ,String page,String charts,String author,String type ,String content,String top,String url) {
        Map<String,Object> usermap=(Map<String, Object>) request.getSession().getAttribute("usermap");
        Map<String, Object> remap =new HashMap<String, Object>();
        Map<String, Object> map =new HashMap<String, Object>();
        if(usermap!=null&&!usermap.isEmpty()){
            remap.put("islog", true);
        }else{
            remap.put("islog", false);
            return remap;
        }
        String filePath = null;
//		String uploadPath = null;
        InputStream is = null;
        InputStream is2 = null;
        try {
            String fileExtension = FileUtil.getFileExtension(file.getOriginalFilename());
            String fileExtension2 = FileUtil.getFileExtension(filepic.getOriginalFilename());
            is = file.getInputStream();
            is2 = filepic.getInputStream();
//			uploadPath = "/5/cpsb/ADCBS/adcbs-ui/html/upload";
            String path = iFileStore.storeFile(is, fileExtension, filePath, basePath);
            String path2 = iFileStore.storeFile(is2, fileExtension2, filePath, basePath);
            map.put("filename", FileUtil.getFileName(file.getOriginalFilename()) );
            map.put("path", "/ADC/upload/"+path);
            map.put("cover", "/ADC/upload/"+path2);
            map.put("title", title );
            map.put("page", page );
            map.put("charts", charts);
            map.put("author",author );
            map.put("type", type);
            map.put("up_user", usermap.get("USERNAME"));
            map.put("top",top );
            map.put("content",content );
            map.put("url",url );
            int i=reportService.upPdf(map);
            remap.put("isup", i);
        } catch (IOException e) {
            logger.error(e.getMessage(), e);
            remap.put("isup", 0);
            return remap;
        } finally {
            IOUtils.closeQuietly(is);
        }
        return remap;
    }


    /**
     * 文件上传
     * @param file 指定文件上传路径，空则在根路径
     * pdf_info新增
     * @return
     */
    @SuppressWarnings("unchecked")
    @ApiOperation(value = "|FileEO|上传")
    @PostMapping("/uploadPDF2")
    public Map<String, Object> uploadPDF(HttpServletRequest request,
            @RequestParam("filepic") MultipartFile filepic,String title
            ,String author,String type ,String content,String top,String url,String contentdetail) {
        Map<String,Object> usermap=(Map<String, Object>) request.getSession().getAttribute("usermap");
        Map<String, Object> remap =new HashMap<String, Object>();
        Map<String, Object> map =new HashMap<String, Object>();
       /* if(usermap!=null&&!usermap.isEmpty()){
            if("61077".equals(usermap.get("ID").toString()) && "adc".equals(usermap.get("USERNAME").toString() )){
                remap.put("islog", true);
            }else{
                remap.put("islog", false);
                return remap;
            }

        }else{
            remap.put("islog", false);
            return remap;
        }*/
        String filePath = null;
//		String uploadPath = null;
        InputStream is = null;
        InputStream is2 = null;
        try {

            String fileExtension2 = FileUtil.getFileExtension(filepic.getOriginalFilename());

            is2 = filepic.getInputStream();
//			uploadPath = "/5/cpsb/ADCBS/adcbs-ui/html/upload";

            String path2 = iFileStore.storeFile(is2, fileExtension2, filePath, basePath);
            map.put("filename", "");
            map.put("path", "");
            map.put("cover", "/ADC/upload/"+path2);
            map.put("title", title );
            map.put("page", "" );
            map.put("charts", "");
            map.put("author",author );
            map.put("type", type);
            map.put("up_user", usermap.get("USERNAME"));
            map.put("top",top );
            map.put("content",content );
            map.put("url",url );
            map.put("contentdetail",contentdetail );
            int i=reportService.upPdf(map);
            remap.put("isup", i);
        } catch (IOException e) {
            logger.error(e.getMessage(), e);
            remap.put("isup", 0);
            return remap;
        } finally {
            IOUtils.closeQuietly(is);
        }
        return remap;
    }

    /**
     * 上传文件
     * 上传的文件仅发生在以下几个页面：
     * 1： 分析报告页面中的 "公告"栏
     * 2：分析报告页面中的 "新能源"栏
     * 3：政策研究页面
     * 4：资源共享页面(1级菜单)中的 会议资料(2级菜单)
     *
     * @param request
     * @param fileUploadQueryVO
     * @return java.util.Map<java.lang.String                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                               ,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                               java.lang.Object>
     * @author 韩磊
     * @date 2019/3/20 16:37
     */
    @SuppressWarnings("unchecked")
    @ApiOperation(value = "|FileEO|上传")
    @PostMapping("/uploaFile")
    public ResponseMessage uploadFile(HttpServletRequest request, @RequestParam("file") MultipartFile file,
            @RequestParam("pic") MultipartFile pic, FileUploadQueryVO fileUploadQueryVO) {
        Map<String, Object> usermap = (Map<String, Object>) request.getSession().getAttribute("usermap");

        if (null == usermap || usermap.isEmpty()) {
            return Result.error("请先登录系统后再进行申请.");
        }

        if (null == file && StringUtils.isBlank(file.getOriginalFilename())) {
            return Result.error("请上传文件.");
        }
        FileTypeEnum fileTypeEnum = FileTypeEnum.getFileEnumByValue(fileUploadQueryVO.getType());
        if (null == fileTypeEnum) {
            return Result.error("请重新选择服务类型.");
        }
        int n = analyzeReportService.uploadFile(file,pic,fileUploadQueryVO,usermap);
        if (0 != n) {
            return Result.success();
        } else {
            return Result.error("系统忙，请稍后再试.");
        }
    }
}